!
! hbnddia --- diagonalizes the band matrix computed by the
!       complex Hermitian band Lanczos
! Hans-Dieter Meyer
!
!
!     diagonalizes the bandmatrix computed by the block lanczos
!     and purge the spectrum
!
!     this is not difficult but the purging and output selection
!     is quite complex; in addition there are many fossile structures
!     of Dieter or Uwe and some of the aux arrays are used for several
!     purposes
!
!     n        matrix dimension = no. of lanczos iterations
!     np       half band width  = blocksize in lanczos
!     t        bandmatrix
!     z        contains the first and last np components
!              of the eigenvectors of t
!     a,e,e2   aux arrays
!     d        contains the eigenvalues of t
!     thresh1  threshold for eigenvalue output
!     thresh2  threshold for output of coefficients
!     thresh1 > thresh2 is assumed
!
!     the following parameters are needed to compute eigenvectors.
!     in the 1st version all eigenvectors of t were computed for
!     this purpose and z(n,n); in this version the eigenvectors are
!     computed using inverse iteration and z(np2,n), i.e., zdim is
!     always np2
!
!     neigenv          no. of eigenvectors
!     evlist           is the list of eigenvectors to be computed;
!                      if evlist(0)=0 the eigenvectors are selected
!                      via a max overlap criterion (default)
!     au, al, b, ipvt  are aux arrays for the inverse iteration
!                      they are not refered if neigenv = 0
!     iparam is an arry with flags
!     iparam(1) = dieterout (output in dieters format)
!
!     the old output of dieter (first and last n eigenvalues) is
!     controlled via dieterout; my output (all eigenvalues with a
!     certain overlap with the startspace) is controlled via
!     thresh1
!     -------------------------------------------------------------
!
      subroutine hbnddia(n, np, t, a, d, e, e2, z, thresh,              &
     &                   neigenv, evlist, au, al, b, ipvt,              &
     &                   iparam, ps, numStates)
      use physconst
      implicit none

      integer maxn, maxnp, numStates
      parameter (maxn = 10000, maxnp = 500)

      integer n, np, neigenv, evlist(neigenv), ipvt(n), iparam(2)
      ! CB complex*16 t(np+1,n), a(n,np+1), e(n), e2(n), z(2*np, n)
      complex(dblprec) t(np+1,n), a(np+1, n), e(n), e2(n), z(2*np, n)
      complex(dblprec) au(n,2*np+1), al(n,2*np+1), b(n)
      ! CB real*8 thresh(3), d(n)
      real(dblprec) thresh(3), d(n), ps(n)

      integer np1, np2, zdim, mmm, i, j, iwgt(maxnp), nwgt
      integer ninvi, ind2(maxn), indx, ngt, wrterf
      real(dblprec) wgt(maxnp)
      real(dblprec) thresh1, thresh2, eta, r
!
!     Dieters Programmteile (war vorher implizit deklariert)
!
      integer dieterout
      integer nsp, nmu, isp, jnd, ind(maxn), mlt(maxn), lsp(maxn)
      integer k, kk, ia, ic, jc
      complex(dblprec) aa, sum, sup, ak, a1k, xm
      real(dblprec) ovl, thrs, y1, x1

! CB
      integer info
      real(dblprec), allocatable :: rwork(:)
      complex(dblprec), allocatable :: work(:), Tz(:,:)

      external zdotc, zhbev
      complex(dblprec) zdotc

#include "iosetting.inc"
!
!     ---------------------------------------------------------------
      np1 = np + 1
      np2 = 2 * np
      mmm = np2 + 1
      zdim = np2
      ninvi = 1
      dieterout = iparam(1)
      wrterf = iparam(2)
      thresh1 = thresh(1)
      thresh2 = thresh(2)
      eta     = thresh(3)

      write (6,4000)
 4000 format (//'  Diagonalization of the band-matrix T'/)
      write (6,*) '    matrix dimension : ', n
      write (6,*) '    half band width  : ', np
      if (n .gt. maxn .or. np .gt. maxnp) then
         write(6,*)
         write(6,*) 'maxdim = ', maxn
         write(6,*) 'maxnp  = ', maxnp
         write(6,*) 'change parameters in banddia.'
         stop
      end if


        ! do not transpose, just copy
      do i = 1, n
         do j = 1, np1
            a(j,i) = t(j,i)
         end do
      end do

/**************************************************************

       The lower triangle of T is stored in packed form

!     ----------------------------------------------
!     t in bandrc Format umspeichern (transponieren)
!     z could be used for this purpose, but t is considered
!     to be small in comparison to the lanczos blocks
!     ----------------------------------------------
      do i = 1, n
         do j = 1, np1
            a(i,j) = t(j,i)
         end do
      end do



! Complex symmetric code
!
!     zbandr is called anyway to initialize z
!
      call zbandr (zdim, n, np, a, .true., z)
!
!     rearrange the tridiagonal matrix
!
      d(1) = a(1,1)
      e(1) = (0.0d0, 0.0d0)
      do i = 2, n
         d(i) = a(i,1)
         e(i) = a(i-1,2)
      end do

      call ztql2 (zdim, n, d, e, z, i)
      if (i.ne.0) then
         write (6,*) '====  Error in tql2  ==='
         write (6,*) i,                                                 &
     &        '-th Eigenvalue not converged after 30 Iterations'
         stop
      end if
**************************************************************/


      ! Use Lapack to diagonalize the Hermitian band matrix
      allocate(work(n), rwork(max(1, 3 * n - 2)), Tz(n, n))
      call zhbev('V', 'L', n, np, a, np+1, d, Tz, n, work, rwork, info)
      if (info /= 0) then
        stop 'zhbev error!'
      end if
      z(1:np,1:n)  = Tz(1:np,1:n)
      z(np+1:2*np,1:n) = Tz(n-np+1:n,1:n)
      deallocate(work, rwork, Tz)


!
!     write the first unpurged eigenvalues to stdout
!
      write (6,'(/,A)') '  The lowest 10 eigenvalues of T:'
      do i = 1, 10
         write (6,3000) i, d(i) * HartreetoeV
 3000    format (4x,I3,'  (',F15.8,',',F15.8,')')
      end do

!
!     T has been diagonalized, now purge the spectrum and
!     prepare a nice output
!
!     ----------------------------------------------------------
!     DETERMINE THE INDEX-VECTOR 'IND' IN ORDER TO REMOVE
!     MULTIPLE AND SPURIOUS EIGENVALUES.
!     ----------------------------------------------------------
      NSP = 0
      NMU = 0
      ISP = 0
      JND = 0           ! Counter of useful eigenvalues
      J   = 0           ! Counter of eigenvalues of the bandmatrix T

      ! Loop over all eigenvalues of the bandmatrix T
  50  J   = J + 1
      IF( J .GT. n ) then
        GOTO 40
      end if
!      CALL zMULT( Z(1,J), Z(1,J), NP , AA )
! CB      aa=zdotu(np,z(1,j),1,z(1,j),1)
!     r = |z(1,j)|**2
      r = real(zdotc(np, z(1,j), 1, z(1,j), 1))
      aa = r
      NSP = NSP + 1
      ISP = ISP + 1
!.....TEST OF SPURIOUS EIGENVALUE.
      IF( abs(r) .LT. 1.D-14 ) then
        GOTO  50
      end if
      NSP = NSP - 1
      ISP = ISP - 1
      JND = JND + 1
      IND(JND) = J
      MLT(JND) = 0
      LSP(JND) = ISP
      ISP      = 0

      IF( J .EQ. n ) then
        GOTO  40
      end if
      THRS = 5.D-7*( 1.D0 + ABS(d(J)) )
!.....TEST OF MULTIPLE EIGENVALUE.
      IF( ABS( d(J) - d(J+1) ) .GT. THRS ) then
        GOTO  50
      end if
      SUM = AA
      SUP = AA
      KK  = 0
      IA  = 0

      DO  41  K = 1, n - J
      OVL = 1.D-5
      IF( ABS( d(J+IA) - d(J+K) ) .GT. THRS ) then
        GOTO  42
      end if
!      CALL zMULT( Z(1,J+K), Z(1,J+K), NP , AK )
! CB      ak = zdotu(np,z(1,j+k),1,z(1,j+k),1)
      ! r = |z(1,j+k)|**2
      r = zdotc(np, z(1,j+k), 1, z(1,j+k), 1)
      ak = r
      IF( abs(r) .LE. 1.D-14 )  THEN
         KK     = K
         GOTO  41
      ELSE IF( abs(AK/SUP + SUP/AK)  .LT.  2.D+6 )    THEN
           OVL = 1.D-7
           IF( ABS(d(J+IA)-d(J+K)) .GT. 1.D-2*THRS ) then
             GOTO  42
           end if
      END IF
!      CALL zMULT( Z(1,J+K), Z(1,J+IA), NP , A1K )
! CB      a1k = zdotu(np,z(1,j+k),1,z(1,j+ia),1)
      a1k = zdotc(np, z(1,j+k), 1, z(1,j+ia), 1)
      A1K = A1K*A1K/(SUP*AK)
      IF( ABS( A1K-1.D0 ) .LE. OVL )            THEN
         SUM = SUM + AK
         IF( abs(AK) .GT. abs(SUP) )  THEN
           SUP = AK
           IA  =  K
         END IF
         KK = K
      ELSE
        GOTO  42
      END IF
  41  CONTINUE

  42  d(J) = d(J+IA)
      DO  I = 1, NP
         Z(   I,J) = Z(   I,J+IA) * SQRT(SUM/SUP)
         Z(NP+I,J) = Z(NP+I,J+IA)
      end do

      NMU = NMU + KK
      J   =  J  + KK
      MLT(JND) =  KK
      GOTO  50

  40  CONTINUE

!
!.....REMOVE SPOURIOUS AND MULTIPLE EIGENVALUES.
!
      DO J = 1, JND
         I = IND(J)
         d(J) = d(I)
         DO  K = 1, NP2
            Z(K,J) = Z(K,I)
         end do
      end do

      write (6,'(/,A)') '  Information on purging:'
      PRINT 905, JND, n, int(100.0d0*dble(jnd)/dble(n))
 905  FORMAT(4x,'There are',I5,' relevant residues out of',I5,          &
     &     '   (',I3,'%)')
      write(6,'(4x,A,I4,6x,A,I4)') 'N_spurios = ', NSP,                 &
     &     'N_multiple = ', NMU
      ! CB number of proper states (not spurious, not multiple)
      numStates = jnd
!
!
!.....TRANSFORM THE EIGENVECTORS(FIRST NP ROWS) TO OBTAIN THE RESIDUES.
!.....STORE THE RESULT IN THE FIRST NP ROWS OF THE EIGENVECTOR MATRIX Z.
!.....MULTIPLY :  EIGENVECTORS(LAST ROWS) * 'ERROR-TRIANGLE'
!.....STORE THIS RESULT IN THE LAST ROWS OF THE EIGENVECTOR MATRIX Z.
!
      DO J = 1, JND             ! CB inserted
       DO I  = 1, NP
         XM = (0.0d0, 0.0d0)
         DO K = I, NP
           KK = K + NP
           IC = K + n - NP
           JC = I - K + NP1
           XM = XM + Z(KK,J) * T(JC,IC)
         end do
         Z(I+NP,J) = XM
       end do
      end do                    ! CB inserted


! Insert dieterout here

!     ------------------------------------------------------------
!     print eigenvalues mit mean-res > thresh1
!     collect the corresponding indices in ind2()
!     and the mean res in the array e2  (aux for bandrm)
!     --------------------------------------------------
! CB      write(6,400) thresh1
 400  format(//2x,68('-')/'  Eigenvalues with mean-res > ',F6.3,/       &
     &     /'   no.  mean-res.       eigenvalue',16x,'error estimate'/2x&
     &     ,68('-'))
      ngt = 0
      do j = 1, jnd
         x1 = 0.d0
         do i = 1, np
            x1 = x1 + dble(z(i,j) * conjg(z(i,j)))
         end do
         ! CB polestrengths zurckgeben (Tarantelli definition)
         ps(j) = X1
         x1 = sqrt(x1)
         if (x1 .gt. thresh1) then
            ngt = ngt + 1
            ind2(ngt) = ind(j)
            e2(ngt) = cmplx(x1, 0.0d0, dblprec)
            y1 = 0.0d0
            do i = 1, np
               y1 = y1 + dble(z(np+i,j) * conjg(z(np+i,j)))
            end do
            y1 = sqrt(y1)
         ! CB   write(6,420) j, x1, d(j) * HartreetoeV, y1, ind(j)
!! 420        format(I5,2x,F8.4,'  (',F14.8,',',1PE16.8,')',1PE11.2,3x,'('
!      !!!&           ,I4,')')
         end if
      end do
!     ---------------------------------------------------------
!     write the eigenvalue list to a separate file for eta-runs
!     ---------------------------------------------------------
      if (wrterf .eq. 1) then
         open (ifile,file='ETARUN',access='SEQUENTIAL',form='FORMATTED')
         write(ifile,'(1PE18.8)') eta
         write(ifile,'(I6)') ngt
         do j = 1, ngt
            write(ifile,'(I4,1PE18.8,1PE18.8)') j, d(ind2(j))
         end do
         close (ifile)
      end if
!     ---------------------------------------------------------
!     compute eigenvectors of Z using invers iteration
!     the eigenvectors are written to disk: ZMAT
!     if evlist is undefined the first neigenv in ind2 are used
!     ---------------------------------------------------------
      if (neigenv .gt. 0) then
         if (evlist(1) .eq. 0) then
            if (neigenv .gt. ngt) then
               write(6,*) 'Only ', ngt, ' vectors can be computed.'
               neigenv = ngt
            end if
            do i = 1, neigenv
               evlist(i) = ind2(i)
            end do
         end if
         write(6,430) (evlist(i), i = 1, neigenv)
 430     format(/,'  The following Eigenvectors are computed:'/,        &
     &        20(4x,10(2x,I5),/))
         open (ifile,file='ZMAT',access='SEQUENTIAL',form='UNFORMATTED')
         write(ifile) n, neigenv
         do j = 1, neigenv
            do i = 1, n
               b(i) = (0.0d0, 0.0d0)
            end do
            do i = 1, np
               b(i)          = z(i, evlist(j))
               b(n - np + i) = z(np + i, evlist(j))
            end do
            indx = 0
            do i = 1, jnd
               if (evlist(j) .eq. ind(i)) then
                  indx=i
               end if
            end do
            if (indx .eq. 0) then
               write(6,*) 'Index ', evlist(j), ' has not been found.'
               stop
            end if
! CB            call zinvit(n,np,np1,mmm,d(indx),t,au,al,b,ipvt,                &
! CB    &           ninvi,.false.)
           stop 'no invit!'
            write(ifile) (b(i), i = 1, n)
         end do
         close(ifile)
      end if
!     -----------------------------------------------------
!     print weights of start vectors for mean-res > thresh2
!     -----------------------------------------------------
      if (thresh2 .lt. 1.0d0) then
         write(6,450) thresh2
 450     format(/2x,60('-')/'  absolute weights (|C|**2) for m-r >',F6.3&
     &        ,/)
         do i = 1, jnd
            x1 = 0.D0
            do j = 1, np
               x1 = x1 + dble(z(j,i) * conjg(z(j,i)))
            end do
            x1 = dsqrt(x1)
            if (x1 .gt. thresh2) then
               nwgt = 0
               do j = 1, np
                  x1 = dble(z(j,i) * conjg(z(j,i)))
                  if (x1 .gt. 0.001d0) then
                     nwgt = nwgt + 1
                     iwgt(nwgt) = j
                     wgt(nwgt) = x1
                  end if
               end do
               write(6,470) i, d(i) * HartreetoeV
 470           format(1X,I4,'-th root  E = (',F14.8,',',F14.8,')')
               write(6,480) (wgt(j), iwgt(j), j=1, nwgt)
 480           format(6(4x,8(2x,F5.3,'(',I2,')'),/))
               write(6,*)
            end if
         end do
      end if
!
!.. ...PRINT MEAN THE 'ERROR TRIANGLE' OF THE T-MATRIX.
!
!      PRINT 940
! 940  FORMAT(//2X,'" E R R O R - T R I A N G L E "  (TRANSPOSED)'/
!     #       /2X,125('-')//)
!      DO  333  II = 1, NP
!         PRINT 941, (T(NP1+K-II,n+1-K),K=1,II)
! 941  FORMAT(1X, 10F12.6 )
!         IF( II .GE. 10 ) PRINT *
! 333  CONTINUE

      end subroutine hbnddia


/**************************************
!
!.....PRINT MEAN RESIDUES (X), EIGENVALUES (W), ERROR-EST.(Y) AND MULT.
!
      if (dieterout .eq. 0) GOTO 329
      ICNV = 0
      JCNV = 0
      PRINT 900
 900  FORMAT(//2X,'E I G E N V A L U E S'/2X,125('-')//2                &
     &     ('   NO.  MEAN-RES.*100  EIGEN-VALUE',16x                    &
     &     ,'ERROR     M   S',3x) // 2X,125('-')/ )
      DO  330  J = 1, (JND+1)/2
         Y1 = 0.D0
         Y2 = 0.D0
         K  = JND + 1 - J
         DO  332  I = 1, NP
            Y1 = Y1 + dble(Z(NP+I,J) * dconjg(z(np+i,j)))
            Y2 = Y2 + dble(Z(NP+I,K) * dconjg(z(np+i,k)))
 332     continue
         Y1 = dsqrt(Y1)
         Y2 = dsqrt(Y2)
         IF(Y1.LT.1.D-9 .AND. J.NE.JND/2+1) ICNV = ICNV + 1
         IF(Y2.LT.1.D-9)  ICNV = ICNV + 1
         IF(Y1.LT.1.D-6 .AND. J.NE.JND/2+1) JCNV = JCNV + 1
         IF(Y2.LT.1.D-6)  JCNV = JCNV + 1
         IF( J .GT. dieterout )   GOTO  330
         X1 = 0.D0
         X2 = 0.D0
         DO  331  I = 1, NP
            X1 = X1 + dble(Z(I,J) * dconjg(z(i,j)))
            X2 = X2 + dble(Z(I,K) * dconjg(z(i,k)))
 331     continue
         X1 = DSQRT(X1)
         X2 = DSQRT(X2)
         PRINT 901, J, X1, d(J) * HartreetoeV, Y1, MLT(J), LSP(J), K,   &
     &              X2, d(K) * HartreetoeV, Y2, MLT(K)                  &
     &        ,LSP(K)
 901     FORMAT (2(I5,F8.4,' (',F14.8,',',E15.7,')',E9.2,I3,1X,I3))
 330  CONTINUE
      PRINT 990
 990  FORMAT(/2('   NO.  MEAN-RES.      EIGEN-VALUE',16x,               &
     &'ERROR     M   S', 4X) // 2X,125('-')/ )
      PRINT 902, ICNV, 100.D0*ICNV/DBLE(JND), 100.D0*ICNV/DBLE(n)
      PRINT 903, JCNV, 100.D0*JCNV/DBLE(JND), 100.D0*JCNV/DBLE(n)
 902  FORMAT(/,'    NUMBER OF CONVERGED (1.D-9) EIGENVALUES =',         &
     &         I4, F11.2, ' %  (JND) ', F9.2,' %  (JEND) ' )
 903  FORMAT(/,'    NUMBER OF CONVERGED (1.D-6) EIGENVALUES =',         &
     &         I4, F11.2, ' %  (JND) ', F9.2,' %  (JEND) ' )
 329  CONTINUE
**************************************/