!
! calcSESW --- Assemble the full Hamiltonian
! Christian Buth
! 23 May 2006
!


      subroutine calcSESW(zHmat, SESfTfatm, SESpotatm, SESWatom, dimen, nphmin, &
     &          nphmax, lmin, lmax, nvec)
      use floquetmat
      use matrixfile
      implicit none

      integer, intent(in) :: dimen, nphmin, nphmax, lmin, lmax, nvec
      complex(dblprec), intent(inout) :: zHmat(dimen,dimen)
      complex(dblprec), intent(in) :: SESfTfatm(nvec,nvec,lmin:lmax), &
     &                                SESpotatm(nvec,nvec,lmin:lmax), &
     &                                SESWatom (nvec,nvec,lmin:lmax)

      integer indr, indc, mur, muc, lr, lc, i, j
      type(tmatrixfile) mf

      ! SES W is diagonal in mu and l
      indc = 0
      do muc = nphmin, nphmax
        do lc = lmin, lmax
          do j = 1, nvec
            indc = indc + 1
            indr = 0
            do mur = nphmin, nphmax
              do lr = lmin, lmax
                do i = 1, nvec
                  indr = indr + 1
                  if (muc == mur .and. lc == lr) then
                    zHmat(indr,indc) = zHmat(indr,indc) + SESfTfatm(i,j,lc)
                    zHmat(indr,indc) = zHmat(indr,indc) + SESpotatm(i,j,lc)
                    zHmat(indr,indc) = zHmat(indr,indc) + SESWatom (i,j,lc)
                  end if
                end do
              end do
            end do
          end do
        end do
      end do


      ! Open files for the diagonal and the off-diagonal matrix
      ! elements of the CO-ADC matrix
      mf%readfile     = .false.
      mf%fmtxcom      = 393
      mf%fmtxdia      = 394
      mf%fmtxoff      = 395
      mf%main         = 10
      mf%ndim         = dimen
      mf%nbuf         = 10000
      mf%threshold    = 0
      call openmatrixfile(mf)


      ! Write upper triangle including the diagonal of zHmat to disk
      indc = 0
      do muc = nphmin, nphmax
        do lc = lmin, lmax
          do j = 1, nvec
            indc = indc + 1
            indr = 0
            do mur = nphmin, nphmax
              do lr = lmin, lmax
                do i = 1, nvec
                  indr = indr + 1
                  if (zHmat(indr, indc) /= (0.0D0, 0.0D0) .and. indr <= indc) then
                    call writeentry(mf, indr, indc, zHmat(indr, indc))
                  end if
                end do
              end do
            end do
          end do
        end do
      end do

      call closematrixfile(mf)
      end subroutine calcSESW


!
! calcZham --- assemble the full complex-symmetric Hamiltonian
!

      subroutine calcZham(dimen, nphmin, nphmax, lmin, lmax, nvec, mi,   &
     &          cconst, eval, omegaL, SESdipatm, SESfTfatm,  &
     &          SESpotatm, SESWatom, Eground)
      use floquetmat
      implicit none

      integer, intent(in) :: dimen, nphmin, nphmax, lmin, lmax, nvec, mi
      real(dblprec), intent(in) :: cconst, omegaL, Eground,  &
     &          eval(nvec,lmin:lmax)
      complex(dblprec), intent(in) :: SESWatom(nvec,nvec,lmin:lmax),  &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),             &
     &          SESpotatm(nvec,nvec,lmin:lmax),                       &
     &          SESfTfatm(nvec,nvec,lmin:lmax)

      integer indc, indr
      complex(dblprec) zHmat(dimen,dimen)

      zHmat(:,:) = 0

      ! Assemble the Hamiltonian
      call calcHamMat(dimen, nphmin, nphmax, lmin, lmax, nvec,          &
     &          mi, cconst, zHmat, eval, Eground, omegaL, SESdipatm)
      call calcSESW(zHmat, SESfTfatm, SESpotatm, SESWatom, dimen, &
     &          nphmin, nphmax, lmin, lmax, nvec)

      ! Check for complex symmetry of the Hamiltonian
      do indc=1, dimen
        do indr=1, indc-1
          if (zHmat(indc,indr) /= zHmat(indr,indc)) then
            stop 'zHmat is NOT complex symmetric!'
          end if
        end do
      end do
      end subroutine calcZham
