!
! KEmtrx --- represent the kinetic energy in terms of finite
!       element basis functions
! Kevin Christ, Mark Baertschy, Christian Buth
! 9 June 2006
!


      subroutine KEmtrx(T,x)
      use FiniteElements
      implicit none

!   type(FEBnodes) :: x
!   INTEGER :: nmnds, nbasis, left, right, row1, j, k, l
  integer nmnds, nbasis, left, right, row1, j, k, l
  REAL(kind=8) :: smT(0:2,0:2), opT(0:2,0:2), h1, h2
! CB  REAL(kind=8) :: T(-5:5,*)
!   real(kind=8) :: T(-5:5,1:nbasis)
!       real(kind=8), intent(inout) :: T(:,:)
      type(FEBnodes), intent(in) :: x
      real(kind=8), intent(inout) :: T(-5:5,1:numbasfun(x))
! END CB


  nmnds = x%numnds
  left = x%numleft
  right = x%numright

  nbasis = numbasfun(x)
  if (nbasis <= 0) RETURN
  T(-5:5,1:nbasis) = 0.0D0

! overlap for half-basis functions "centered" on same node
  smT(0:2,0:2) = RESHAPE( SOURCE = (/ &
    & 5.d0/7.d0, -3.d0/28.d0, 1.d0/168.d0, &
    &-3.d0/28.d0, 4.d0/35.d0,-1.d0/120.d0, &
    & 1.d0/168.d0,-1.d0/120.d0, 1.d0/1260.d0 &
    & /), SHAPE = (/3, 3/) )
! overlap for half-basis functions "centered" on adjacent nodes
  opT(0:2,0:2) = RESHAPE( SOURCE = (/ &
    & -5.d0/7.d0, -3.d0/28.d0, -1.d0/168.d0, &
    &  3.d0/28.d0, -1.d0/140.d0, -1.d0/420.d0, &
    & -1.d0/168.d0, 1.d0/420.d0, 1.d0/2520.d0 &
    & /), SHAPE = (/3, 3/) )


! half-basis functions "centered" at nodes(1)
  row1 = left-2
  h2 = node_spacing(x,1)
  DO j = (3-left),2
    DO k = j,2
      T(j-k,row1+k) = -(-h2)**(j+k-1)*smT(j,k)
    END DO
  END DO

! overlaps between nodes(1) and nodes(2)
  h1 = h2
  IF (nmnds > 2) THEN
    DO j = (3-left),2
      DO k = 0,2
        T(j-k-3,row1+3+k) = h1**(j+k-1)*opT(j,k)
      END DO
    END DO
  ELSE
    DO j = (3-left),2
      DO k = (3-right),2
        T(j-k-left,row1+left+k) = h1**(j+k-1)*opT(j,k)
      END DO
    END DO
  ENDIF

  h1 = h2
! basis functions centered at nodes(2)
  IF (nmnds > 2) THEN
    row1 = left + 1
    h2 = node_spacing(x,2)
    DO j = 0,2
      DO k = j,2
        T(j-k,row1+k) = (h1**(j+k-1)-(-h2)**(j+k-1))*smT(j,k)
      END DO
    END DO
  ENDIF

  DO l = 3,(nmnds-1)
! overlaps between nodes(l-1) and nodes(l)
    h1 = h2
    DO j = 0,2
      DO k = 0,2
        T(j-k-3,row1+3+k) = h1**(j+k-1)*opT(j,k)
      END DO
    END DO

  ! basis functions centered on nodes(l)
    row1 = row1 + 3
    h2 = node_spacing(x,l)
    DO j = 0,2
      DO k = j,2
        T(j-k,row1+k) = (h1**(j+k-1)-(-h2)**(j+k-1))*smT(j,k)
      END DO
    END DO
  END DO

! overlaps between nodes(nmnds-1) and nodes(nmnds)
  IF (nmnds > 2) THEN
    h1 = h2
    DO j = 0,2
      DO k = (3-right),2
        T(j-k-right,row1+right+k) = h1**(j+k-1)*opT(j,k)
      END DO
    END DO
  END IF

! half-basis functions "centered" on nodes(nmnds)
  row1 = row1 + right
  DO j = (3-right),2
    DO k = j,2
      T(j-k,row1+k) = h1**(j+k-1)*smT(j,k)
    END DO
  END DO

  DO k = 1,(nbasis-1)
    DO j = (k+1),MIN(nbasis,k+5)
      T(j-k,k) = T(k-j,j)
    END DO
  END DO

END SUBROUTINE KEmtrx
