SUBROUTINE Wmtrx( Wbig, nstate, nl, egfxn, ngrd, xh, c )

  implicit none

!! Wmtrx calculates the matrix of the CAP operator W(r) in eigenfunction 
!! representation, r being the radial coordinate. The integrals are evaluated 
!! upon the substitution x=sqrt(r) (r is distributed on a quadratic mesh, 
!! x on a linear one). xh is the spacing of the linear mesh.
!!          / 
!!         |  0        , r .le. c
!! W(r) = < 
!!         |  (r-c)**2 , r .gt. c
!!          \

  INTEGER :: nstate, nl, ngrd 
  REAL(kind=8) :: Wbig(nstate,nstate,nl), egfxn(ngrd,nstate,nl)
  REAL(kind=8) :: xh, c, rint
  REAL(kind=8), ALLOCATABLE :: work(:)
  REAL(kind=8) :: x,aux
  INTEGER :: i, j, k, l
  
  ALLOCATE( work(ngrd) )

  aux = dsqrt(c)

  do l = 1, nl
     do k = 1, nstate
        do j = 1, k
           do i = 1, ngrd
              x = xh*dble(i)
              if (x .le. aux) then
                 work(i) = 0.d0
              else
                 work(i) = x*egfxn(i,j,l)*egfxn(i,k,l)
                 work(i) = work(i)*(x**2-c)**2
              end if
           end do
           Wbig(j,k,l) = 2.d0*rint(work,1,ngrd,10,xh)
           Wbig(k,j,l) = Wbig(j,k,l) 
        end do
     end do
  end do

  RETURN
END SUBROUTINE Wmtrx

